import json
import pathlib
import re
from unittest.mock import patch

from lxml import etree
import pytest
from ncclient.manager import make_device_handler, Manager
from ncclient.transport import SSHSession
from ncclient.xml_ import NCElement

from inventory_provider import juniper


TEST_DATA_DIR = pathlib.Path(__file__).parent / "../data"


@pytest.fixture
def classifier_cache_test_entries():
    file = TEST_DATA_DIR.joinpath('classifier-cache-entries.json')
    return json.loads(file.read_text())


@pytest.fixture(params=list(TEST_DATA_DIR.glob('*-netconf.xml')))
def router(request):
    file: pathlib.Path = request.param
    m = re.match('(.*)-netconf.xml', file.name)
    assert m  # sanity
    return m.group(1)


class MockedJunosRpc(object):

    def __init__(self, hostname):
        file = TEST_DATA_DIR.joinpath(f'{hostname}-netconf.xml')
        self.config = etree.parse(file)

    def get_config(self):
        return self.config


class MockedJunosDevice(object):

    def __init__(self, **kwargs):
        self.rpc = MockedJunosRpc(kwargs['host'])

    def open(self):
        pass

    def close(self):
        pass


@pytest.fixture
def netconf_doc(mocker, router, data_config):
    mocker.patch(
        'inventory_provider.juniper.Device',
        MockedJunosDevice)
    return juniper.load_config(router, data_config['ssh'])


# different library requires different mocking

@pytest.fixture
def interface_info_response(router):
    file = TEST_DATA_DIR / f"interface_info/{router}.xml"

    try:
        return file.read_text()
    except FileNotFoundError:
        pytest.skip(f'no corresponding interface_info doc for {router}, skipping')


@pytest.fixture
def mocked_router(interface_info_response):
    def _mocked_manager_connect(*args, **kwargs):
        assert kwargs
        device_params = kwargs["device_params"]
        device_handler = make_device_handler(device_params)
        session = SSHSession(device_handler)
        return Manager(session, device_handler)

    def _mocked_rpc_reply(*args, **kwargs):
        device_handler = make_device_handler({'name': 'junos'})
        rpc_reply = NCElement(
            interface_info_response,
            device_handler.transform_reply()
        )
        return rpc_reply

    with patch('ncclient.manager.connect') as manager:
        manager.side_effect = _mocked_manager_connect
        with patch('ncclient.operations.third_party.juniper.rpc.ExecuteRpc._request') as rpc:
            rpc.side_effect = _mocked_rpc_reply
            with patch('ncclient.operations.session.CloseSession.request'):
                yield
